/***************************************************************************
                          kvoulumerendererview.h  -  description
                             -------------------
    begin                : Mon Jan 21 11:14:26 CET 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KVOULUMERENDERERVIEW_H
#define KVOULUMERENDERERVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt
#include <qwidget.h>
#include <qscrollview.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qcolor.h>
#include <qsize.h>
#include <qpoint.h>

// application specific includes
#include "mypoly3d.h"
#include "zbuffer.h"

 // mutex support
#if defined(QT_THREAD_SUPPORT)
#include <qthread.h>
#else
#include "mymutex.h"
#endif

class KvoulumeRendererDoc;

/** The KvoulumeRendererView class provides the view widget for the document instance connected to it and is displayed
 * as a MDI child window in the main view area of the KvoulumeRendererApp class instance. The KvoulumeRendererApp class also has an eventFilter()
 * method that gets installed on every KvoulumeRendererView instance to control events of the type QEvent::Close.	
 * The document connected to the view instance keeps a list of all view that represent the document contents as there
 * can be more than one view. Views get created in KvoulumeRendererApp::createClient() and automatically added to the list of views.
 * The KvoulumeRendererView class inherits QWidget as a base. Another possible inheritance besides specialized widgets could be
 * QMainWindow so that you can easily set up the main area of your view by setting another view
 * as main widget (QMainWindow::setMainWidget() ).
 * NOTE: The close event always has to be empty (DON`T CALL QWidget::closeEvent(e) in closeEvent()) because the installed
 * event filter can only manage a forward implementation. If the QCloseEvent is received by the KvoulumeRendererView, the overwritten
 * event handler has to do nothing as the eventFilter has set accept() or ignore() already. If QWidget::closeEvent() is called
 * again, the default event handler will accept the close event and the window gets destroyed even if the installed eventFilter
 * has set the event to be ignored.	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 1.3 code generation
 */
class KvoulumeRendererView : public QScrollView
{
  Q_OBJECT

  friend KvoulumeRendererDoc;

  public:
    /** Constructor for the view
    	* @param pDoc  your document instance that the view represents. Create a document before calling the constructor
    	* or connect an already existing document to a new MDI child widget.*/
    KvoulumeRendererView(KvoulumeRendererDoc* pDoc, QWidget* parent, const char *name, int wflags);
    /** Destructor for the main view */
    ~KvoulumeRendererView();
		/** returns a pointer to the document connected to the view*/
    KvoulumeRendererDoc *getDocument() const;
    /** gets called to redraw the document contents if it has been modified */
	void update(KvoulumeRendererView* pSender);
    /** contains the implementation for printing functionality and gets called by KvoulumeRendererApp::slotFilePrint() */
    void print(QPrinter *pPrinter);
	 /** getter for the projection type of the view */
    MyPoly3D::tmatrix::viewtransform_t getProjectionType() { return projectiontype; }
	/** setter for the background color */
   void setBackgroundColor(const QColor &backgroundcolor) {this->backgroundcolor = backgroundcolor.rgb(); }
	/** getter for the background color */
	QColor getBackgroundColor() const { return QColor(backgroundcolor); }
	/** getter for the size of the computed image */
	QSize getRenderSize() const { return rendersize; }
	/** setter for the focal length (necessary for the perspectivic projection) */
	void setFocalLength(float focallength);
	/** getter for the focal length (important  for the perspectivic projection) */
	float getFocalLength() const { return focallength; }	
	/** set the size of the rendered image */
	void setRenderSize(const QSize &rendersize);
 	/** getter for the width of the 3D world window   */
	double getWindowWidth() const { return  window_width_half * 2.0;  }
	/** getter for the height of the 3D world window   */
	double getWindowHeight() const { return  window_height_half * 2.0;  }
	/** setter for the materials settings */
	void setMaterial(double ka,double kd,double ks,int exponent) {this->ka=ka;this->kd=kd;this->ks=ks;this->exponent = exponent; }
	/** getter for the ambiental light strength */
	double getAmbient() const { return ka; }
	/** getter for the diffuse reflection strength */
	double getDiffuse() const { return kd; }
	/** getter for the specular reflection strength */
	double getSpecular() const { return ks; }
	/** getter for the highlight sharpness exponent */
	int getHighlightSharpness() const { return exponent; }
 	/** transforms a window x coordinate into a viewport x coordinate */
	int transformWindowX(double x) { return viewport_halfwidth + int(faktorx*x); }
	/** transforms a window y coordinate into a viewport y coordinate */
	int transformWindowY(double y) { return viewport_halfheight - int(faktory*y); }
	/** transforms a viewport x coordinate into a  window x coordinate */
	double transformViewportX(int x) { return (x-viewport_halfwidth)/faktorx; }
	/** transforms a viewport y coordinate into a  window y coordinate */
	double transformViewportY(int y) { return -(y-viewport_halfheight)/faktory; }
	/** saves the rendered image */
	bool saveImage(QString filename, QString extension) {return drawboard.save(filename,extension);}
  	/** getter for the minimal visible density */
  	unsigned char getMinDensity() const { return mindensity; }
  	/** setter  for the minimal visible density */
  	void setMinDensity(unsigned char min) { mindensity = min; }
  	/** getter for the minimal visible density */
  	unsigned char getMaxDensity() const { return maxdensity; }
  	/** setter  for the minimal visible density */
  	void setMaxDensity(unsigned char max) { maxdensity = max; }  	
  protected:
    /** overwritten QWidget::closeEvent() to catch closing views. Does nothing, as the closeEvents for
    * KvoulumeRendererView's are processed by KvoulumeRendererApp::eventFilter(), so this overwitten closeEvent is necessary
    * and has to be empty. Don't overwrite this method !
    */
 	virtual void closeEvent(QCloseEvent* e);
 	/** what to do when getting the focus */
	/** updates internal factors for the window-viewport-transformation */
	void updateScaling();
	void focusInEvent(QFocusEvent *e) { emit  updateProjectionTypeDisplay(projectiontype); QScrollView::focusInEvent(e); }
 	/** changes the projection type name extension of the caption to the current settings */
 	void updateCaption();
 	/** render the image into the backbuffer */
 	void repaintBackbuffer();
 	/** updates the widget by painting from the backbuffer pixmap */
 	void viewportPaintEvent(QPaintEvent *e);
 	void viewportMouseReleaseEvent(QMouseEvent * ) { leftbuttondown = false; }
 	/** No descriptions */
	void viewportMouseMoveEvent(QMouseEvent *e);
  /** No descriptions */
  void initColorTable();
 	/** The document connected to the view, specified in the constructor */
	KvoulumeRendererDoc *doc;
	/** current projection type */
	MyPoly3D::tmatrix::viewtransform_t projectiontype;
	/** stores the base caption (caption without projection type extension */
	QString basecaption;
	/** stores the projection type extensions */
	static const char *projectiontype_names[];
signals: // Signals
  /** No descriptions */
  void updateProjectionTypeDisplay(MyPoly3D::tmatrix::viewtransform_t projectiontype);
public slots: // Public slots
  /** set the projection type of the view */
  void slotSetProjectionType(MyPoly3D::tmatrix::viewtransform_t projectiontype);
  /** set the base caption of the view (caption without projection type name */
  void slotSetBaseCaption(const QString &basecaption);
  /** No descriptions */
  void slotDocSizeChanged();
 protected:
 	/** background color */
 	QRgb backgroundcolor;
 	/** size of the rendered image */
 	QSize rendersize;
 	/** backbuffer pixmap; it contains the content of the drawboard in machine byte order & screen depth */
 	QPixmap backbuffer;
 	/** image which contains our full-color picture */
 	QImage drawboard;
 	/** zBuffer */
 	zbuffer zBuffer;
 	/** Mutex for the backbuffer */
 	QMutex paintmutex;
 	/* focal length (important  for the perspectivic projection) */
 	float focallength;
 	/** half of the current viewport width */
 	int viewport_halfwidth;
 	 /** half of the current viewport height */
 	int viewport_halfheight;
 	/** factor for transforming a 3D world x coordinate into a viewport x coordinate */
 	double faktorx;
 	/** factor for transforming a 3D world x coordinate into a viewport x coordinate */
 	double faktory; 	
 	/** steps in window which equals one step in view (pixel) coordinates */
 	double stepx,stepy;
 	/** strengths of diffuse & specular reflections and ambiental light */
 	double kd,ks,ka;
 	/** sharpness of specular highlights */
 	int exponent;
 	/** half of width of the mapped area / window to the 3D world */
 	double window_width_half;
 	/** half of height of the mapped area / window to the 3D world */
 	double window_height_half;
	/** diagonal length of voxel cube */
	double diag;
	/** front-lower-left edge of voxel cube in global world coordinates */
	MyPoly3D::point4f P0;
	/** elementary vectors of voxel cube's coordinate system in global world's coordinates */
	MyPoly3D::point4f dw,dh,dd;
 	/** transformation matrix doing the transformation into the camera coordination system */
 	MyPoly3D::tmatrix cameratransform;
 	/** transformation matrix doing the view transform */
 	MyPoly3D::tmatrix viewtransform;
 	/** transformation matrix combining camera and view transformation */
 	MyPoly3D::tmatrix combinedtransform;
 	/** stores status of left mouse button */
 	bool leftbuttondown;
 	/** stores last mouse position */
 	QPoint lastmouseposition;
 	// dimensions of the voxel cube
 	int voxelcubewidth;
	int voxelcubeheight;
	int voxelcubedepth;
	// minimal visible density
	unsigned char mindensity;
	// maximal visible density
	unsigned char maxdensity;
protected slots: // Protected slots
  /** No descriptions */
  void viewportMousePressEvent(QMouseEvent *e);
};

#endif // KVOULUMERENDERERVIEW_H
