/***************************************************************************
                          rawfilesettingsdlg.h  -  description
                             -------------------
    begin                : Mon Jan 21 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RAWFILESETTINGSDLG_H
#define RAWFILESETTINGSDLG_H

#include <qwidget.h>
#include <rawfilesettingsdialog.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qstring.h>

/**dialog configuring a raw file in/export
  *@author Ruediger Knoerig
  */

class RawFileSettingsDlg : public RawFileSettingsDialog  {
   Q_OBJECT
public: 
	RawFileSettingsDlg(QWidget *parent=0, const char *name=0);
	~RawFileSettingsDlg();
	/** formats */
	enum format_t { unsigned8bit,unsigned16bit };
	enum mapping_t { useMSBs };
	/** setter for the voxel space width */
	void setWidth(int width) {  WidthSpin->setValue(width); }
	/** setter for the voxel space height */
	void setHeight(int height) {  HeightSpin->setValue(height); }
	/** setter for the voxel space depth */
	void setDepth(int depth) {  DepthSpin->setValue(depth); }
	/** getter for  the voxel space width */
	int getWidth() { return  WidthSpin->value(); }
	/** getter for  the voxel space height */
	int getHeight() { return  HeightSpin->value(); }
	/** getter for  the voxel space depth */
	int getDepth() { return  DepthSpin->value(); }	
	/** getter for the endian setting */
	bool getBigEndian() { return BigEndianRadio->isChecked(); }
	/** getter for the sample format */
	RawFileSettingsDlg::format_t getFormat() { return (RawFileSettingsDlg::format_t)  FormatCombo->currentItem(); }
	/** getter for the mapping */
	RawFileSettingsDlg::mapping_t getMapping() { return (RawFileSettingsDlg::mapping_t)  MappingCombo->currentItem(); }
	/** setter  for the sample format */
	void setFormat(RawFileSettingsDlg::format_t format) {  FormatCombo->setCurrentItem((int) format); }
	/** setter  for the mapping */
	void setMapping(RawFileSettingsDlg::mapping_t mapping) {  FormatCombo->setCurrentItem((int) mapping); }
	/** setter for a title string */
	void setTitle(const QString &title) { infotext->setText(title); }
protected slots: // Protected slots
  /** gives a size (memory usage) hint to the user */
  void slotGiveSizeHint();
};

#endif
