/***************************************************************************
                          viewpropertydialog.h  -  description
                             -------------------
    begin                : Thu Jan 10 2002
    copyright            : (C) 2002 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VIEWPROPERTYDIALOG_H
#define VIEWPROPERTYDIALOG_H

#include <qwidget.h>
#include <qcolor.h>
#include <qsize.h>
#include <qspinbox.h>
#include <kcolorbutton.h>
#include <knuminput.h>
#include "ViewPropertiesDialog.h"

/**Dialog for configuring the actual view
  *@author Rdiger Knrig
  */

class ViewPropertyDialog : public ViewPropertyDialogTemplate  {
   Q_OBJECT
public:
	/** default constructor */
	ViewPropertyDialog(QWidget *parent=0, const char *name=0);
	/** default destructor */
	~ViewPropertyDialog();
	/** setter for the backgound color property */
	void setBackgroundColor(const QColor  &backgroundcolor) {   BackgroundColorSelector->setColor(backgroundcolor); }
	/** getter for the background color property */
	QColor getBackgroundColor() const { return  BackgroundColorSelector->color(); }
	/** setter for the rendered image size */
	void setRenderedSize(const QSize &rendersize) { renderSizeWidthSpin->setValue(rendersize.width());  renderSizeHeightSpin->setValue(rendersize.height()); }
	/** getter for the rendered image size */
	QSize getRenderedSize() { return QSize(renderSizeWidthSpin->value(),renderSizeHeightSpin->value()); }
	/** setter for the specular strength */
	void getSpecularStrength(double ks)  { SpecularStrengthInput->setValue(ks);  }
	/** setter for the diffuse strength */
	void getDiffuseStrength(double kd)  { DiffuseStrengthInput->setValue(kd);  }
	/** getter for the specular strength */
	double getSpecularStrength() const { return SpecularStrengthInput->value();  }
	/** getter for the diffuse strength */
	double getDiffuseStrength() const { return DiffuseStrengthInput->value();  }
	/** setter for the focal length */
	void setFocalLength(float focallength) { FocalLengthInput->setValue(focallength); }
	/** getter for the focal length */
	float getFocalLength() const { return (float)  FocalLengthInput->value(); }
	/** setter for the minimal vivable  density */
	void setMinDensity(int density) {MinimalDensityInput->setValue(density);}
	/** getter  for the minimal vivable  density */
	unsigned char getMinDensity() const { return (unsigned char)  MinimalDensityInput->value(); }
	/** setter for the minimal vivable  density */
	void setMaxDensity(int density) {MaxDensityInput->setValue(density);}
	/** getter  for the minimal vivable  density */
	unsigned char getMaxDensity() const { return (unsigned char)  MaxDensityInput->value(); }	
	/** setter for the material properties */
	void setMaterialProperties(double ka,double kd,double ks,int exponent) {  AmbientalStrengthInput->setValue(ka);  DiffuseStrengthInput->setValue(kd); SpecularStrengthInput->setValue(ks); HighlightExponentSpin->setValue(exponent); }
	/** get ambiental light strength */
	double getAmbient() const { return   AmbientalStrengthInput->value(); }
	/** get diffuse reflection strength */
	double getDiffuse() const { return   DiffuseStrengthInput->value(); }
	/** get specular reflection strength */
	double getSpecular() const { return SpecularStrengthInput->value(); }	
	/** get specular highlight exponent  */
	int getExponent() const { return HighlightExponentSpin->value(); }	
protected slots: // Protected slots
  /** No descriptions */
  void slotCheckDensities();
};

#endif
