/***************************************************************************
                          voulumeimporter.cpp  -  description
                             -------------------
    begin                : Mon Jan 21 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "voulumeimporter.h"
#include "klocale.h"
#include "kvoulumerendererdoc.h"
#include "rawfilesettingsdlg.h"
#include "kurl.h"
#include <kio/netaccess.h>
#include <qfile.h>
#include <qstring.h>
#include <qdatastream.h>
#include <qtextstream.h>

VoulumeImporter::VoulumeImporter(KvoulumeRendererDoc *doc)
{
	docobj = doc;
}

bool VoulumeImporter::import(const KURL &url,QString &log)
{
	KURL infofileurl(url.url()+".INFO");
	QString infofiletmpfilename;
	KIO::NetAccess::download( infofileurl, infofiletmpfilename );
	QFile infofile( infofiletmpfilename );
	// confirm informations via dialog
	RawFileSettingsDlg dlg;
	if(infofile.open( IO_ReadOnly))
		{
			dlg.setTitle(i18n("I've found an .INFO file but have a close look at the settings!"));
			QString linebuffer;
			QTextStream filestream(&infofile);
			filestream.skipWhiteSpace();
			while(!filestream.eof())
			{
				linebuffer = filestream.readLine();
				linebuffer.stripWhiteSpace();
				if(linebuffer.at(0).isDigit()) // evaluate only lines which start with a number
				{
					if(linebuffer.contains("Schichten",false) || linebuffer.contains("Layer",false))
					{
						QTextStream parsestream(&linebuffer,IO_ReadOnly);
						int depth;
						parsestream >> depth;
						dlg.setDepth(depth);
					}
					if(linebuffer.contains("Graustufen",false) || linebuffer.contains("Colors",false))
					{
						QTextStream parsestream(&linebuffer,IO_ReadOnly);
						int greyvalues;
						parsestream >> greyvalues;
						switch(greyvalues)
						{
						   case 256 : dlg.setFormat(RawFileSettingsDlg::unsigned8bit);
						   		break;
						   	case 65 : // some people could try 65k...
						   	case 65536 : dlg.setFormat(RawFileSettingsDlg::unsigned16bit);
						   		break;
						};
					}		
					if(linebuffer.contains("Groesse",false) || linebuffer.contains("dimension",false))
					{
					  	QTextStream parsestream(&linebuffer,IO_ReadOnly);
					  	int width,height;
					  	QChar tmp;
					  	parsestream >> width >> tmp >> height;
					  	dlg.setWidth(width);
					  	dlg.setHeight(height);
					}	
				}		
			}
			infofile.close();
		}
		else
		{
			dlg.setTitle(i18n("I've found no .INFO file so have to specify the settings manually!"));
		}
			
	KIO::NetAccess::removeTempFile( infofiletmpfilename );
	
	if(dlg.exec() ==   RawFileSettingsDlg::Accepted)
	{
		if(!docobj->resize(dlg.getWidth(),dlg.getHeight(),dlg.getDepth()))
		{
			log += i18n("Could not allocate so much memory!");
			return false;
		}
		QString tmpfile;
  		KIO::NetAccess::download( url, tmpfile );

		QFile f( tmpfile );
		if ( !f.open( IO_ReadOnly ) )
			{
				log += i18n("downloading of data file failed!");
				return false;
			}
		docobj->datamutex.lock();
		unsigned char *ptr = docobj->getPtr();
		QDataStream streamin(&f);
		streamin.setByteOrder((dlg.getBigEndian()) ? QDataStream::BigEndian : QDataStream::LittleEndian );
		int x,y,z;
		int depth  = docobj->getDepth();
		int  height = docobj->getHeight();
		int width = docobj->getWidth();
		switch(dlg.getFormat())
		{
			case RawFileSettingsDlg::unsigned8bit :
				 for(z=0; z < depth; z++)
				 	for(y=0; y < height; y++)
				 		for(x=0; x < width && !streamin.eof();x++,ptr++) streamin >> *ptr;
				 break;
			case RawFileSettingsDlg::unsigned16bit :
				Q_INT16 temp;
				switch(dlg.getMapping())
				{
					case  RawFileSettingsDlg::useMSBs :
						
						for(z=0; z < depth; z++)
				 			for(y=0; y < height; y++)
				 				for(x=0; x < width && !streamin.eof();x++,ptr++)
				 				{
				 					streamin >> temp;
				 					temp >>= 8;
				 					*ptr = (unsigned char) (temp & 0xFF);
				 				}
				};	
		};
		docobj->datamutex.unlock();			
		f.close();
		KIO::NetAccess::removeTempFile( tmpfile );
		if(z == depth && y == height  && x == width)
			{
				  log += i18n("sucessfully imported ") + QString::number(width*height*depth) + i18n(" voxels!");
				  return true;
			}
			else
			{
				QString temp;
				log += i18n("expected ") + temp.sprintf(" %d x %d x %d ",width,height,depth) +  i18n(" voxels\nbut got end of file at (x,y,z) ");
				log += temp.sprintf(" %d x %d x %d ",x,y,z);
				return false;
			}
	}
	return false;	
}